package org.acme;

import static jakarta.interceptor.Interceptor.Priority.APPLICATION;

import jakarta.annotation.Priority;
import jakarta.decorator.Decorator;
import jakarta.decorator.Delegate;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.enterprise.inject.Any;
import jakarta.inject.Inject;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;
import org.jboss.logging.Logger;

interface Repository<T> {
  T get();
}

@Priority(APPLICATION)
@Decorator
class LoggingRepository<T> implements Repository<T> {

  @Inject Logger logger;

  @Inject @Any @Delegate Repository<T> delegate;

  @Override
  public T get() {
    logger.debug("Getting item");
    return delegate.get();
  }
}

@ApplicationScoped
class GreetingRepository implements Repository<String> {

  @Override
  public String get() {
    return "Hello from RESTEasy Reactive";
  }
}

@Path("/hello")
class GreetingResource {

  @Inject GreetingRepository repository;

  @GET
  @Produces(MediaType.TEXT_PLAIN)
  public String hello() {
    return repository.get();
  }
}
